﻿using System.Collections.ObjectModel;
using System.Diagnostics; // Add this for Debug.WriteLine
using System.Windows.Input;
using Microsoft.Maui.Controls;
using RobobinApp.Views;
using Plugin.BLE;
using Plugin.BLE.Abstractions.Contracts;
using Plugin.BLE.Abstractions.Exceptions;
using Plugin.BLE.Abstractions.EventArgs;


namespace RobobinApp.ViewModels
{
    public class ConnectionPageViewModel : BaseViewModel
    {
        private BluetoothDevice _selectedDevice;
        public ICommand GoHomeCommand { get; }
        public ObservableCollection<BluetoothDevice> BluetoothDevices { get; }
        public ICommand ConnectCommand { get; }

        // BLE plugin fields
        private readonly IAdapter _adapter;
        private readonly IBluetoothLE _bluetoothLE;

        public ConnectionPageViewModel()
        {
            BluetoothDevices = new ObservableCollection<BluetoothDevice>();
            ConnectCommand = new Command(OnConnect);
            GoHomeCommand = new Command(async () => await OnGoHome());

            // Initialize BLE manager and adapter
            _bluetoothLE = CrossBluetoothLE.Current;
            _adapter = _bluetoothLE.Adapter;

            // Subscribe to the DeviceDiscovered event
            _adapter.DeviceDiscovered += OnDeviceDiscovered;

            // Check and request Bluetooth permissions before scanning
            Debug.WriteLine("Checking and requesting Bluetooth permissions.");
            CheckAndRequestBluetoothPermissions();
        }

        public BluetoothDevice SelectedDevice
        {
            get => _selectedDevice;
            set
            {
                _selectedDevice = value;
                OnPropertyChanged();
            }
        }

        private async void CheckAndRequestBluetoothPermissions()
        {
            // Check if the Bluetooth scan permission is granted
            Debug.WriteLine("Checking Bluetooth permissions.");
            var status = await Permissions.CheckStatusAsync<Permissions.Bluetooth>();
            if (status != PermissionStatus.Granted)
            {
                Debug.WriteLine("Bluetooth permission not granted, requesting permission.");
                // Request permission
                status = await Permissions.RequestAsync<Permissions.Bluetooth>();
            }

            if (status == PermissionStatus.Granted)
            {
                Debug.WriteLine("Bluetooth permission granted, proceeding to scan devices.");
                // Permission granted, proceed with scanning devices
                ScanDevices();
            }
            else
            {
                // Permission denied, handle accordingly
                Debug.WriteLine("Bluetooth permission denied.");
                await Application.Current.MainPage.DisplayAlert("Permissions", "Bluetooth scan permission is required to discover devices.", "OK");
            }
        }

        public async Task OnGoHome()
        {
            Debug.WriteLine("Navigating to home page.");
            await Application.Current.MainPage.Navigation.PushAsync(new MainPage());
        }

        private async void ScanDevices()
        {
            // Clear the current list of devices before scanning
            BluetoothDevices.Clear();
            Debug.WriteLine("Cleared existing Bluetooth devices. Starting scan...");

            try
            {
                if (!_bluetoothLE.IsAvailable)
                {
                    Debug.WriteLine("Bluetooth is not available.");
                    await Application.Current.MainPage.DisplayAlert("Bluetooth", "Bluetooth is not available.", "OK");
                    return;
                }

                if (!_bluetoothLE.IsOn)
                {
                    Debug.WriteLine("Bluetooth is turned off.");
                    await Application.Current.MainPage.DisplayAlert("Bluetooth", "Please turn on Bluetooth.", "OK");
                    return;
                }

                // Start scanning for devices
                Debug.WriteLine("Starting scanning for devices...");
                await _adapter.StartScanningForDevicesAsync();
            }
            catch (Exception ex)
            {
                Debug.WriteLine($"Error during scanning: {ex.Message}");
                await Application.Current.MainPage.DisplayAlert("Error", $"Failed to scan for devices: {ex.Message}", "OK");
            }
        }

        // New method to handle discovered devices
        private void OnDeviceDiscovered(object sender, DeviceEventArgs e)
        {
            Debug.WriteLine($"Discovered device: ID={e.Device.Id}, Name={e.Device.Name}, State={e.Device.State}, RSSI={e.Device.Rssi}");


            var bluetoothDevice = new BluetoothDevice
            {
                Name = string.IsNullOrWhiteSpace(e.Device.Name) ? e.Device.Id.ToString() : e.Device.Name,
                MacAddress = e.Device.Id.ToString()
            };

            if (!BluetoothDevices.Any(d => d.MacAddress == bluetoothDevice.MacAddress))
            {
                Device.BeginInvokeOnMainThread(() =>
                {
                    BluetoothDevices.Add(bluetoothDevice);
                    Debug.WriteLine($"Added device to list: {bluetoothDevice.Name} ({bluetoothDevice.MacAddress})");
                });
            }
        }




        private async void OnConnect()
        {
            if (SelectedDevice == null)
            {
                Debug.WriteLine("No device selected for connection.");
                return;
            }

            // Logic to connect to the selected Bluetooth device
            Debug.WriteLine($"Attempting to connect to {SelectedDevice.Name}...");
            await Application.Current.MainPage.DisplayAlert("Connection", $"Connecting to {SelectedDevice.Name}...", "OK");

            try
            {
                // Stop scanning before attempting connection
                await _adapter.StopScanningForDevicesAsync();
                Debug.WriteLine("Stopped scanning for devices.");

                // Find the device using the MAC address
                var device = await _adapter.ConnectToKnownDeviceAsync(Guid.Parse(SelectedDevice.MacAddress));

                // Display successful connection
                Debug.WriteLine($"Successfully connected to {SelectedDevice.Name}.");
                await Application.Current.MainPage.DisplayAlert("Connected", $"Connected to {SelectedDevice.Name}.", "OK");
            }
            catch (DeviceConnectionException ex)
            {
                // Handle connection failure
                Debug.WriteLine($"Connection error: {ex.Message}");
                await Application.Current.MainPage.DisplayAlert("Connection Error", $"Failed to connect: {ex.Message}", "OK");
            }
            catch (Exception ex)
            {
                Debug.WriteLine($"General connection error: {ex.Message}");
                await Application.Current.MainPage.DisplayAlert("Connection Error", $"An error occurred: {ex.Message}", "OK");
            }
        }
    }

    public class BluetoothDevice
    {
        public string Name { get; set; }
        public string MacAddress { get; set; }
    }
}